# -*- coding: utf-8 -*-

from iugu.action import Action
from iugu.exception import RequiredParameters


class Invoice(Action):

    def create(self, data):
        if not data.get('email', None):
            raise RequiredParameters('Invoice email not informed')
        elif not data.get('due_date', None):
            raise RequiredParameters('Invoice due_date not informed')
        elif not data.get('items', None):
            raise RequiredParameters('Invoice items not informed')
        url = self.api.make_url(['invoices'])
        return super(Invoice, self).create(url, data)

    def search(self, id):
        url = self.api.make_url(['invoices', id])
        return super(Invoice, self).search(url)

    def change(self, id, data):
        if not data.get('email', None):
            raise RequiredParameters('Invoice email not informed')
        elif not data.get('due_date', None):
            raise RequiredParameters('Invoice due_date not informed')
        elif not data.get('items', None):
            raise RequiredParameters('Invoice items not informed')
        url = self.api.make_url(['invoices', id])
        return super(Invoice, self).change(url, data)

    def remove(self, id):
        url = self.api.make_url(['invoices', id])
        return super(Invoice, self).remove(url)

    def cancel(self, id):
        url = self.api.make_url(['invoices', id, 'cancel'])
        return self.api.put(url)

    def refund(self, id):
        url = self.api.make_url(['invoices', id, 'refund'])
        return self.api.post(url)

    def list(self, data={}):
        url = self.api.make_url(['invoices'])
        return super(Invoice, self).list(url, data)



# -*- coding: utf-8 -*-

from iugu.iuguapi import IuguApi, config
from iugu.token import Token
from iugu.customer import Customer, PaymentMethod
from iugu.plan import Plan
from iugu.subscription import Subscription
from iugu.transfer import Transfer
from iugu.marketplace import MarketPlace
from iugu.invoice import Invoice
from iugu.version import __version__



# -*- coding: utf-8 -*-

from iugu.action import Action
from iugu.exception import RequiredParameters


class Subscription(Action):

    def create(self, data):
        if not data.get('customer_id', None):
            raise RequiredParameters('Subscription customer_id not informed')
        url = self.api.make_url(['subscriptions'])
        return super(Subscription, self).create(url, data)

    def search(self, id):
        url = self.api.make_url(['subscriptions', id])
        return super(Subscription, self).search(url)

    def change(self, id, data):
        url = self.api.make_url(['subscriptions', id])
        return super(Subscription, self).change(url, data)

    def remove(self, id):
        url = self.api.make_url(['subscriptions', id])
        return super(Subscription, self).remove(url)

    def suspend(self, id):
        url = self.api.make_url(['subscriptions', id, 'suspend'])
        return self.api.post(url)

    def active(self, id):
        url = self.api.make_url(['subscriptions', id, 'activate'])
        return self.api.post(url)

    def change_plan(self, id, plan_identifier):
        url = self.api.make_url(['subscriptions', id,
                                 'change_plan', plan_identifier])
        return self.api.post(url)

    def add_credits(self, id, quantity):
        data = {'quantity': quantity}
        url = self.api.make_url(['subscriptions', id, 'add_credits'])
        return self.api.post(url, data)

    def remove_credits(self, id, quantity):
        data = {'quantity': quantity}
        url = self.api.make_url(['subscriptions', id, 'remove_credits'])
        return self.api.post(url, data)

    def list(self, data={}):
        url = self.api.make_url(['subscriptions'])
        return super(Subscription, self).list(url, data)



# -*- coding: utf-8 -*-

from iugu.action import Action
from iugu.exception import RequiredParameters


class Transfer(Action):

    def create(self, data):
        if not data.get('receiver_id', None):
            raise RequiredParameters('Transfer receiver_id not informed')
        if not data.get('amount_cents', None):
            raise RequiredParameters('Transfer amount_cents not informed')
        url = self.api.make_url(['transfers'])
        return super(Transfer, self).create(url, data)

    def list(self, data={}):
        url = self.api.make_url(['subscriptions'])
        return super(Transfer, self).list(url, data)



# -*- coding: utf-8 -*-

from iugu.action import Action
from iugu.exception import RequiredParameters


class Customer(Action):

    def create(self, data):
        if not data.get('email', None):
            raise RequiredParameters('Customer email not informed')
        url = self.api.make_url(['customers'])
        return super(Customer, self).create(url, data)

    def search(self, id):
        url = self.api.make_url(['customers', id])
        return super(Customer, self).search(url)

    def change(self, id, data):
        url = self.api.make_url(['customers', id])
        return super(Customer, self).change(url, data)

    def remove(self, id):
        url = self.api.make_url(['customers', id])
        return super(Customer, self).remove(url)

    def list(self, data={}):
        url = self.api.make_url(['customers'])
        return super(Customer, self).list(url, data)


class PaymentMethod(Action):

    def __init__(self, customer_id):
        self.customer_id = customer_id
        self.base_url_paths = ['customers', self.customer_id,
                               'payment_methods']
        super(PaymentMethod, self).__init__()

    def create(self, data):
        if not data.get('token', None) and not data.get('data', {}):
            raise RequiredParameters('Please, inform token or card data{} to create a Payment Method')
        elif not data.get('description', None):
            raise RequiredParameters('Payment Method description not informed')
        url = self.api.make_url(self.base_url_paths)
        return super(PaymentMethod, self).create(url, data)

    def search(self, id):
        url = self.api.make_url(self.base_url_paths + [id])
        return super(PaymentMethod, self).search(url)

    def change(self, id, description):
        data = {'description': description}
        url = self.api.make_url(self.base_url_paths + [id])
        return super(PaymentMethod, self).change(url, data)

    def remove(self, id):
        url = self.api.make_url(self.base_url_paths + [id])
        return super(PaymentMethod, self).remove(url)

    def list(self):
        url = self.api.make_url(self.base_url_paths)
        return super(PaymentMethod, self).list(url)



# -*- coding: utf-8 -*-

from iugu.action import Action
from iugu.exception import RequiredParameters


class Plan(Action):

    def create(self, data):
        if not data.get('name', None):
            raise RequiredParameters('Plan name not informed')
        elif not data.get('identifier', None):
            raise RequiredParameters('Plan identifier not informed')
        elif not data.get('interval', None):
            raise RequiredParameters('Plan interval not informed')
        elif not data.get('interval_type', None):
            raise RequiredParameters('Plan interval_type not informed')
        elif not data.get('currency', None):
            raise RequiredParameters('Plan currency not informed')
        elif not data.get('value_cents', None):
            raise RequiredParameters('Plan value_cents not informed')
        url = self.api.make_url(['plans'])
        return super(Plan, self).create(url, data)

    def search(self, id=None, identifier=None):
        if not id and not identifier:
            raise RequiredParameters('Please, inform the id or identifier of the plan')
        if id:
            url = self.api.make_url(['plans', id])
        else:
            url = self.api.make_url(['plans', 'identifier', identifier])
        return super(Plan, self).search(url)

    def change(self, id, data):
        url = self.api.make_url(['plans', id])
        return super(Plan, self).change(url, data)

    def remove(self, id):
        url = self.api.make_url(['plans', id])
        return super(Plan, self).remove(url)

    def list(self, data={}):
        url = self.api.make_url(['plans'])
        return super(Plan, self).list(url, data)



# -*- coding: utf-8 -*-

from iugu.action import Action
from iugu.exception import RequiredParameters


class Token(Action):

    def create(self, data):
        if not data.get('account_id', None):
            raise RequiredParameters('Token account_id not informed')
        elif not data.get('method', None):
            raise RequiredParameters('Token method not informed')
        elif not data.get('data', None):
            raise RequiredParameters('Token data{} not informed')
        url = self.api.make_url(['payment_token'])
        return super(Token, self).create(url, data)

    def charge(self, data):
        url = self.api.make_url(['charge'])
        return super(Token, self).create(url, data)



# -*- coding: utf-8 -*-

from iugu.iuguapi import default_api


class Action(object):

    def __init__(self):
        self.api = default_api()

    def create(self, url, data):
        return self.api.post(url, data)

    def search(self, url):
        return self.api.get(url)

    def change(self, url, data):
        return self.api.put(url, data)

    def remove(self, url):
        return self.api.delete(url)

    def list(self, url, data={}):
        return self.api.get(url, data)



# -*- coding: utf-8 -*-

import os
import re
import json
import requests
from iugu import exception
from iugu.version import __version__
from requests.auth import HTTPBasicAuth

class IuguApi(object):

    def __init__(self, **kwargs):
        self.token = kwargs.get("token")

    def headers(self):
        return {
            "User-Agent": "Iugu Python Api %s" % __version__,
            "Content-Type": "application/json",
            "Accept": "application/json"
        }

    def base_request(self, url, method, data={}):
        try:
            response = requests.request(method, url,
                                        auth=HTTPBasicAuth(self.token, ''),
                                        data=json.dumps(data),
                                        headers=self.headers())
            return json.loads(response.content.decode('utf-8'))
        #TODO: Create especifics exceptions
        except Exception as error:
            raise

    def get(self, url, data={}):
        return self.base_request(url, 'GET', data=data)

    def post(self, url, data={}):
        return self.base_request(url, 'POST', data=data)

    def put(self, url, data={}):
        return self.base_request(url, 'PUT', data=data)

    def delete(self, url):
        return self.base_request(url, 'DELETE')

    def make_url(self, paths):
        url = 'https://api.iugu.com/v1/'
        for path in paths:
            url = re.sub(r'/?$', re.sub(r'^/?', '/', str(path)), url)
        return url

__default_api__ = None


def default_api():

    global __default_api__
    if __default_api__ is None:
        try:
            token = os.environ["IUGU_API_TOKEN"]
        except KeyError:
            raise exception.ConfigError("Required IUGU_API_TOKEN")
        __default_api__ = IuguApi(token=token)
    return __default_api__


def config(**kwargs):

    global __default_api__
    __default_api__ = IuguApi(**kwargs)
    return __default_api__



# -*- coding: utf-8 -*-

try:
    from distutils.core import setup
except ImportError:
    from setuptools import setup

import sys, os
sys.path.insert(0, os.path.join(os.path.dirname(__file__), 'iugu'))
from version import __version__

setup(
  name='iugu',
  version= __version__,
  author='Felipe Tomaz, Arthur Furlan',
  author_email='gmail@felipetomaz.com, afurlan@afurlan.org',
  packages=['iugu'],
  scripts=[],
  url='https://github.com/iugu/iugu-python',
  license='MIT',
  description='The Iugu provides a Python REST APIs to create, process and manage payments.',
  long_description="""
    The Iugu provides a Python REST APIs to create, process and manage payments.

    http://iugu.com/referencias/api - API Reference
  """,
  install_requires=['requests'],
  classifiers=[
    'Intended Audience :: Developers',
    'Natural Language :: English',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 2.6',
    'Programming Language :: Python :: 2.7',
    'Topic :: Software Development :: Libraries :: Python Modules'
  ],
  keywords=['iugu', 'rest', 'payment']
)


# -*- coding: utf-8 -*-

from iugu.action import Action
from iugu.exception import RequiredParameters


class MarketPlace(Action):

    def create(self, data):
        if not data.get('name', None):
            raise RequiredParameters('MarketPlace name not informed')
        url = self.api.make_url(['marketplace', 'create_account'])
        return super(MarketPlace, self).create(url, data)

    def request_verification(self, id, data):
        if not data.get('data', None):
            raise RequiredParameters('MarketPlace data not informed')
        if not data.get('files', None):
            raise RequiredParameters('MarketPlace files not informed')
        url = self.api.make_url(['accounts', id, 'request_verification'])
        return self.api.post(url, data)

    def sub_account(self, id):
        url = self.api.make_url(['accounts', id])
        return super(MarketPlace, self).list(url)
